/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.common.ui.javafx.layout;

import static org.fortiss.tooling.common.ui.javafx.util.JavaFXUtils.loadFXML;

import java.io.IOException;
import java.net.URL;

import javafx.fxml.FXML;
import javafx.scene.Node;

/**
 * JavaFX node which can be hierarchically composed. It can be included within another
 * {@link CompositeFXControllerBase} and might contain other {@link CompositeFXControllerBase}s.
 * 
 * @author munaro
 */
public abstract class CompositeFXControllerBase implements ICompositeFXController {

	/**
	 * JavaFX {@link Node} with the layout specified in the {@link FXML} resource and
	 * associated with a controller of type {@code T}.
	 */
	private Node layout;

	/** Constructor. */
	public CompositeFXControllerBase(CompositeFXControllerBase... containments) throws IOException {
		URL fxmlResource = getClass().getResource(getFXMLLocation());
		layout = loadFXML(fxmlResource, this);
		addContainments(containments);
	}

	/**
	 * Returns a JavaFX {@link Node} with the layout specified in the {@link FXML} resource and
	 * associated with a controller of type {@code T}.
	 */
	@Override
	public Node getLayout() {
		return layout;
	}

	/** Returns the location of the {@link FXML} resource with the view's layout. */
	@Override
	public abstract String getFXMLLocation();

	/** {@inheritDoc} */
	@Override
	public abstract void addContainments(ICompositeFXController[] containments);
}
